"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TypeGenerator = void 0;
const node_path_1 = __importDefault(require("node:path"));
const tsmodule_1 = __importDefault(require("../tsmodule"));
const errors_1 = require("../errors");
const typegen_1 = require("../typegen");
const string_util_1 = require("../string-util");
const ts_ast_util_1 = require("../ts-ast-util");
class TypeGenerator {
    constructor({ prjRootPath, extractor, tag, addonFactories, debug }) {
        this._prjRootPath = prjRootPath;
        this._extractor = extractor;
        this._tag = tag;
        this._addonFactories = addonFactories;
        this._debug = debug;
        this._printer = tsmodule_1.default.createPrinter({ newLine: tsmodule_1.default.NewLineKind.LineFeed, removeComments: false });
    }
    createAddon({ schema, fileEntry, outputSource, }) {
        const context = {
            schema,
            source: outputSource,
            extractedInfo: {
                fileName: fileEntry.fileName,
                tsTemplateNode: fileEntry.templateNode,
                tsSourceFile: fileEntry.templateNode.getSourceFile(),
            },
        };
        const addons = this._addonFactories.map(factory => factory(context));
        return { addon: (0, typegen_1.mergeAddons)(addons), context };
    }
    generateTypes({ files, schema }) {
        const extractedResult = this._extractor.extract(files, this._tag);
        const extractedErrors = this._extractor.pickupErrors(extractedResult);
        if (extractedErrors.length) {
            this._debug(`Found ${extractedErrors.length} extraction errors.`);
        }
        const typegenErrors = [];
        const visitor = new typegen_1.TypeGenVisitor({ schema });
        const outputSourceFiles = [];
        extractedResult.fileEntries.forEach(fileEntry => {
            if (fileEntry.documentNode) {
                const { inflatedDocumentNode, isDefinedExternal } = this._extractor.inflateDocument(fileEntry, extractedResult);
                const { type, fragmentName, operationName } = this._extractor.getDominantDefinition(fileEntry);
                if (type === 'complex') {
                    const fileName = fileEntry.fileName;
                    const content = fileEntry.templateNode.getSourceFile().getFullText();
                    const start = fileEntry.templateNode.getStart();
                    const end = fileEntry.templateNode.getEnd();
                    const errorContent = { fileName, content, start, end };
                    const error = new errors_1.ErrorWithLocation('This document node has complex operations.', errorContent);
                    typegenErrors.push(error);
                    return;
                }
                const operationOrFragmentName = type === 'fragment' ? fragmentName : operationName;
                if (!operationOrFragmentName)
                    return;
                const outputFileName = node_path_1.default.resolve(node_path_1.default.dirname(fileEntry.fileName), '__generated__', (0, string_util_1.dasherize)(operationOrFragmentName) + '.ts');
                try {
                    const outputSource = (0, ts_ast_util_1.createOutputSource)({ outputFileName });
                    const { addon } = this.createAddon({ schema, outputSource, fileEntry });
                    const outputSourceFile = visitor.visit(inflatedDocumentNode, { outputSource, addon });
                    const content = this._printer.printFile(outputSourceFile);
                    outputSourceFiles.push({ fileName: outputFileName, content });
                    this._debug(`Create type source file '${node_path_1.default.relative(this._prjRootPath, outputFileName)}' from '${node_path_1.default.relative(this._prjRootPath, fileEntry.fileName)}'.`);
                }
                catch (error) {
                    if (error instanceof typegen_1.TypeGenError) {
                        if (isDefinedExternal(error.node))
                            return;
                        const sourcePosition = fileEntry.resolevedTemplateInfo.getSourcePosition(error.node.loc.start);
                        if (sourcePosition.isInOtherExpression)
                            return;
                        const fileName = fileEntry.fileName;
                        const content = fileEntry.templateNode.getSourceFile().getFullText();
                        const start = sourcePosition.pos;
                        const end = fileEntry.resolevedTemplateInfo.getSourcePosition(error.node.loc.end).pos;
                        const errorContent = { fileName, content, start, end };
                        const translatedError = new errors_1.ErrorWithLocation(error.message, errorContent);
                        typegenErrors.push(translatedError);
                    }
                    else {
                        throw error;
                    }
                }
            }
        });
        return { errors: [...extractedErrors, ...typegenErrors], outputSourceFiles };
    }
}
exports.TypeGenerator = TypeGenerator;
//# sourceMappingURL=type-generator.js.map