"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.validate = void 0;
const graphql_language_service_1 = require("graphql-language-service");
const errors_1 = require("../errors");
const gql_ast_util_1 = require("../gql-ast-util");
const string_util_1 = require("../string-util");
function calcEndPositionSafely(startPositionOfSource, getSourcePosition, innerPosition) {
    let endPositionOfSource = 0;
    try {
        endPositionOfSource = getSourcePosition(innerPosition).pos;
    }
    catch (error) {
        endPositionOfSource = startPositionOfSource + 1;
    }
    return endPositionOfSource;
}
function validate({ fileEntries: extractedResults, globalFragments }, schema) {
    const errors = [];
    extractedResults.forEach(r => {
        if (!r.resolevedTemplateInfo)
            return;
        const { combinedText, getSourcePosition, convertInnerLocation2InnerPosition } = r.resolevedTemplateInfo;
        const fragmentNamesInText = (0, gql_ast_util_1.getFragmentNamesInDocument)(r.documentNode);
        errors.push(...(0, gql_ast_util_1.getFragmentsInDocument)(r.documentNode)
            .map(fragmentDef => [fragmentDef, getSourcePosition(fragmentDef.name.loc.start)])
            .filter(([fragmentDef, { isInOtherExpression }]) => !isInOtherExpression && globalFragments.duplicatedDefinitions.has(fragmentDef.name.value))
            .map(([fragmentDef, { pos: startPositionOfSource }]) => new errors_1.ErrorWithLocation(errors_1.ERROR_CODES.duplicatedFragmentDefinitions.message, {
            fileName: r.fileName,
            severity: 'Error',
            content: r.templateNode.getSourceFile().getText(),
            start: startPositionOfSource,
            end: calcEndPositionSafely(startPositionOfSource, getSourcePosition, fragmentDef.name.loc.end),
        })));
        const externalFragments = r.documentNode
            ? (0, graphql_language_service_1.getFragmentDependenciesForAST)(r.documentNode, (0, gql_ast_util_1.cloneFragmentMap)(globalFragments.definitionMap, fragmentNamesInText))
            : [];
        const diagnostics = (0, graphql_language_service_1.getDiagnostics)(combinedText, schema, undefined, undefined, externalFragments);
        diagnostics.forEach(diagnositc => {
            try {
                const { pos: startPositionOfSource, isInOtherExpression } = getSourcePosition(convertInnerLocation2InnerPosition(diagnositc.range.start, true));
                if (isInOtherExpression)
                    return;
                const endPositionOfSource = calcEndPositionSafely(startPositionOfSource, getSourcePosition, convertInnerLocation2InnerPosition(diagnositc.range.end));
                errors.push(new errors_1.ErrorWithLocation(diagnositc.message, {
                    fileName: r.fileName,
                    severity: diagnositc.severity === 2 ? 'Warn' : 'Error',
                    content: r.templateNode.getSourceFile().getText(),
                    start: startPositionOfSource,
                    end: endPositionOfSource,
                }));
            }
            catch (e) {
                if (e instanceof string_util_1.OutOfRangeError) {
                    // Note:
                    // We can not convertInnerLocation2InnerPosition if semantics diagnostics are located in externalFragments.
                    // In other words, there is no error in the original sanitized template text, so nothing to do.
                    return;
                }
                throw e;
            }
        });
    });
    return errors;
}
exports.validate = validate;
//# sourceMappingURL=validator.js.map