"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FragmentRegistry = exports.DefinitionFileStore = void 0;
const graphql_1 = require("graphql");
const graphql_language_service_1 = require("graphql-language-service");
const cache_1 = require("../cache");
const utility_functions_1 = require("./utility-functions");
function union(...sets) {
    return new Set(sets.map(s => [...s.values()]).flat());
}
function intersect(a, b) {
    const [x, y] = a.size < b.size ? [new Set(a), new Set(b)] : [new Set(b), new Set(a)];
    const ret = new Set();
    for (const v of x.values()) {
        if (y.has(v)) {
            ret.add(v);
            x.delete(v);
            y.delete(v);
        }
    }
    if (a.size < b.size) {
        return [ret, x, y];
    }
    else {
        return [ret, y, x];
    }
}
function compare(a, b) {
    if (a.size !== b.size)
        return false;
    let result = true;
    for (const key of a.keys()) {
        result && (result = b.has(key));
    }
    return result;
}
function groupBy(arr, keyPropName) {
    const map = new Map();
    for (const item of arr) {
        const hit = map.get(item[keyPropName]);
        if (hit) {
            hit.push(item);
        }
        else {
            map.set(item[keyPropName], [item]);
        }
    }
    return map;
}
class DefinitionFileStore {
    constructor({ parse, enabledDebugAssertInvaliant = false, }) {
        this._storeVersion = 0;
        this._affectedDefinitonNameHistories = [];
        this._mapByFileName = new Map();
        this._uniqueRecordMap = new Map();
        this._duplicatedRecordMap = new Map();
        this._parse = parse;
        this._enabledDebugAssertInvaliant = enabledDebugAssertInvaliant;
    }
    assertInvariant() {
        if (intersect(new Set(this._uniqueRecordMap.keys()), new Set(this._duplicatedRecordMap.keys()))[0].size > 0) {
            throw new Error('There should be no intersected keys between _duplicatedRecordMap and _uniqueRecordMap');
        }
        if (!compare(new Set([...this._mapByFileName.values()].flat()), union(new Set(this._uniqueRecordMap.values()), new Set([...this._duplicatedRecordMap.values()].flat())))) {
            throw new Error('All values in _mapByFileName should appear _uniqueRecordMap or _duplicatedRecordMap');
        }
    }
    getStoreVersion() {
        return this._storeVersion;
    }
    getDetailedAffectedDefinitions(from) {
        return this._affectedDefinitonNameHistories.slice(from);
    }
    getSummarizedAffectedDefinitions(from) {
        return union(...this._affectedDefinitonNameHistories.slice(from).flatMap(x => [x.appeared, x.disappeared, x.updated]));
    }
    getUniqueDefinitonMap() {
        return this._uniqueRecordMap;
    }
    getDuplicatedDefinitonMap() {
        return this._duplicatedRecordMap;
    }
    updateDocuments(fileName, documents) {
        var _a;
        this._assert();
        if (!documents.length && !this._mapByFileName.has(fileName))
            return;
        const currentValues = (_a = this._mapByFileName.get(fileName)) !== null && _a !== void 0 ? _a : [];
        const currentValueMapByText = groupBy(currentValues, 'documentString');
        const valuesNotChanged = [];
        const valuesToBeAdded = [];
        for (const doc of documents) {
            const alreadyParsedItems = currentValueMapByText.get(doc.text);
            if (alreadyParsedItems && alreadyParsedItems.length === 1) {
                alreadyParsedItems[0].extra = doc.extra;
                valuesNotChanged.push(alreadyParsedItems[0]);
                currentValueMapByText.delete(doc.text);
                continue;
            }
            for (const parsed of this._parse(doc.text)) {
                valuesToBeAdded.push({
                    fileName,
                    extra: doc.extra,
                    documentString: doc.text,
                    definitionName: parsed.name,
                    node: parsed.node,
                });
            }
        }
        const valuesToBeRemoved = [...currentValueMapByText.values()].flat();
        const { namesRemoved, namesFromDuplicatedToUnique } = this._removeValueFromRecordMaps(...valuesToBeRemoved);
        const { namesAcquiredAsUnique, namesToDuplicatedFromUnique } = this._addValueToRecordMaps(...valuesToBeAdded);
        const [namesUpdated, namesFullyRemoved, namesFullyAppeared] = intersect(namesRemoved, namesAcquiredAsUnique);
        const [, namesRecovered, namesHidden] = intersect(namesFromDuplicatedToUnique, namesToDuplicatedFromUnique);
        const namesAppeared = union(namesFullyAppeared, namesRecovered);
        const namesDisappeared = union(namesFullyRemoved, namesHidden);
        this._affectedDefinitonNameHistories[this._storeVersion] = {
            updated: namesUpdated,
            appeared: namesAppeared,
            disappeared: namesDisappeared,
        };
        this._mapByFileName.set(fileName, [...valuesNotChanged, ...valuesToBeAdded]);
        this._storeVersion++;
        this._assert();
    }
    _assert() {
        if (this._enabledDebugAssertInvaliant) {
            try {
                this.assertInvariant();
            }
            catch (e) {
                /* eslint-disable no-console */
                console.log('_mapByFileName:', this._mapByFileName);
                console.log('_uniqueRecordMap:', this._uniqueRecordMap);
                console.log('_duplicatedRecordMap:', this._duplicatedRecordMap);
                /* eslint-enable no-console */
                throw e;
            }
        }
    }
    _addValueToRecordMaps(...values) {
        const namesAcquiredAsUnique = new Set();
        const namesToDuplicatedFromUnique = new Set();
        for (const v of values) {
            const name = v.definitionName;
            const alreadyStoredValuesAsDuplicated = this._duplicatedRecordMap.get(name);
            if (alreadyStoredValuesAsDuplicated) {
                alreadyStoredValuesAsDuplicated === null || alreadyStoredValuesAsDuplicated === void 0 ? void 0 : alreadyStoredValuesAsDuplicated.push(v);
                continue;
            }
            const alreadyStoredValueAsUnique = this._uniqueRecordMap.get(name);
            if (alreadyStoredValueAsUnique) {
                this._uniqueRecordMap.delete(name);
                this._duplicatedRecordMap.set(name, [alreadyStoredValueAsUnique, v]);
                namesToDuplicatedFromUnique.add(name);
            }
            else {
                this._uniqueRecordMap.set(name, v);
                namesAcquiredAsUnique.add(name);
            }
        }
        for (const name of namesAcquiredAsUnique.values()) {
            if (!this._uniqueRecordMap.has(name)) {
                namesAcquiredAsUnique.delete(name);
            }
        }
        return { namesAcquiredAsUnique, namesToDuplicatedFromUnique };
    }
    _removeValueFromRecordMaps(...values) {
        const namesFromDuplicatedToUnique = new Set();
        const namesRemoved = new Set();
        for (const v of values) {
            const name = v.definitionName;
            const alreadyStoredValueAsUnique = this._uniqueRecordMap.get(name);
            if (alreadyStoredValueAsUnique) {
                this._uniqueRecordMap.delete(name);
                if (!namesFromDuplicatedToUnique.has(name)) {
                    namesRemoved.add(name);
                }
                continue;
            }
            const alreadyStoredValuesAsDuplicated = this._duplicatedRecordMap.get(name);
            if (alreadyStoredValuesAsDuplicated && alreadyStoredValuesAsDuplicated.length == 2) {
                const [a, b] = alreadyStoredValuesAsDuplicated;
                const itemToBeKept = a === v ? b : a;
                this._uniqueRecordMap.set(name, itemToBeKept);
                this._duplicatedRecordMap.delete(name);
                namesFromDuplicatedToUnique.add(name);
            }
            else if (alreadyStoredValuesAsDuplicated && alreadyStoredValuesAsDuplicated.length > 2) {
                const s = new Set(alreadyStoredValuesAsDuplicated);
                s.delete(v);
                this._duplicatedRecordMap.set(name, [...s.values()]);
            }
        }
        for (const name of namesFromDuplicatedToUnique.values()) {
            if (!this._uniqueRecordMap.has(name)) {
                namesFromDuplicatedToUnique.delete(name);
            }
        }
        return {
            namesFromDuplicatedToUnique,
            namesRemoved,
        };
    }
}
exports.DefinitionFileStore = DefinitionFileStore;
class FragmentRegistry {
    constructor(options = { logger: () => null }) {
        this._fileVersionMap = new Map();
        this._store = new DefinitionFileStore({
            parse: documentStr => {
                try {
                    const documentNode = (0, graphql_1.parse)(documentStr);
                    return (0, utility_functions_1.getFragmentsInDocument)(documentNode).map(node => ({ node, name: node.name.value }));
                }
                catch {
                    return [];
                }
            },
        });
        this._externalFragmentsCache = new cache_1.LRUCache(100);
        this._logger = options.logger;
    }
    getFileCurrentVersion(fileName) {
        return this._fileVersionMap.get(fileName);
    }
    getFragmentDefinitions() {
        return this.getFragmentDefinitionsWithMap().definitions;
    }
    getFragmentDefinitionsWithMap() {
        const map = new Map([...this._store.getUniqueDefinitonMap().entries()].map(([k, { node }]) => [k, node]));
        const definitions = [...map.values()];
        return {
            map,
            definitions,
        };
    }
    getFragmentDefinitionEntryDetail(name) {
        return this._store.getUniqueDefinitonMap().get(name);
    }
    getDuplicaterdFragmentDefinitions() {
        return new Set(this._store.getDuplicatedDefinitonMap().keys());
    }
    getExternalFragments(documentStr, fileName, sourcePosition) {
        let documentNode = undefined;
        try {
            documentNode = (0, graphql_1.parse)(documentStr);
        }
        catch {
            // Nothing to do
        }
        if (!documentNode)
            return [];
        const names = (0, utility_functions_1.getFragmentNamesInDocument)(documentNode);
        const cacheKey = `${fileName}:${sourcePosition}`;
        const cachedValue = this._externalFragmentsCache.get(cacheKey);
        if (cachedValue) {
            if (compare(new Set(cachedValue.internalFragmentNames), new Set(names))) {
                const changed = this._store.getSummarizedAffectedDefinitions(cachedValue.storeVersion);
                let affectd = false;
                const referencedFragmentNames = new Set();
                (0, graphql_1.visit)(documentNode, {
                    FragmentSpread: node => {
                        affectd || (affectd = changed.has(node.name.value));
                        referencedFragmentNames.add(node.name.value);
                    },
                });
                if (!affectd && compare(referencedFragmentNames, new Set(cachedValue.referencedFragmentNames))) {
                    this._logger('getExternalFragments: use cached value');
                    return cachedValue.externalFragments;
                }
            }
        }
        const map = new Map([...this._store.getUniqueDefinitonMap().entries()]
            .filter(([name]) => !names.includes(name))
            .map(([k, v]) => [k, v.node]));
        const externalFragments = (0, graphql_language_service_1.getFragmentDependenciesForAST)(documentNode, map);
        const referencedFragmentNames = [];
        (0, graphql_1.visit)(documentNode, {
            FragmentSpread: node => {
                referencedFragmentNames.push(node.name.value);
            },
        });
        this._externalFragmentsCache.set(cacheKey, {
            storeVersion: this._store.getStoreVersion(),
            internalFragmentNames: names,
            externalFragments,
            referencedFragmentNames,
        });
        return externalFragments;
    }
    registerDocuments(fileName, version, documentStrings) {
        this._fileVersionMap.set(fileName, version);
        this._store.updateDocuments(fileName, documentStrings.map(({ text, sourcePosition }) => ({ text, extra: { sourcePosition } })));
    }
    removeDocument(fileName) {
        this._store.updateDocuments(fileName, []);
    }
}
exports.FragmentRegistry = FragmentRegistry;
//# sourceMappingURL=fragment-registry.js.map