"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getSemanticDiagnostics = void 0;
const graphql_language_service_1 = require("graphql-language-service");
const tsmodule_1 = __importDefault(require("../tsmodule"));
const errors_1 = require("../errors");
const ts_ast_util_1 = require("../ts-ast-util");
const gql_ast_util_1 = require("../gql-ast-util");
const string_util_1 = require("../string-util");
function createSchemaErrorDiagnostic(errorInfo, file, start, length) {
    let messageText = `[ts-graphql-plugin] Schema build error: '${errorInfo.message}' `;
    if (errorInfo.fileName) {
        messageText += `Check ${errorInfo.fileName}`;
        if (errorInfo.locations && errorInfo.locations[0]) {
            messageText += `[${errorInfo.locations[0].line + 1}:${errorInfo.locations[0].character + 1}]`;
        }
    }
    const category = tsmodule_1.default.DiagnosticCategory.Error;
    const code = errors_1.ERROR_CODES.schemaBuildError.code;
    return { category, code, messageText, file, start, length };
}
function createDuplicatedFragmentDefinitonsDiagnostic(file, sourcePosition, defNode) {
    var _a, _b, _c;
    const startInner = (_b = (_a = defNode.name.loc) === null || _a === void 0 ? void 0 : _a.start) !== null && _b !== void 0 ? _b : 0;
    const length = ((_c = defNode.name.loc) === null || _c === void 0 ? void 0 : _c.end) ? defNode.name.loc.end - startInner : 0;
    return {
        category: tsmodule_1.default.DiagnosticCategory.Error,
        file,
        start: sourcePosition + startInner,
        length,
        messageText: errors_1.ERROR_CODES.duplicatedFragmentDefinitions.message,
        code: errors_1.ERROR_CODES.duplicatedFragmentDefinitions.code,
    };
}
function createIsInOtherExpressionDiagnostic(file, start, length) {
    const category = tsmodule_1.default.DiagnosticCategory.Error;
    const code = errors_1.ERROR_CODES.errorInOtherInterpolation.code;
    const messageText = errors_1.ERROR_CODES.errorInOtherInterpolation.message;
    return { category, code, messageText, file, start, length };
}
function translateDiagnostic(d, file, start, length) {
    const category = d.severity === 2 ? tsmodule_1.default.DiagnosticCategory.Warning : tsmodule_1.default.DiagnosticCategory.Error;
    const code = errors_1.ERROR_CODES.graphqlLangServiceError.code;
    const messageText = d.message.split('\n')[0];
    return { code, messageText, category, file, start, length };
}
function getSemanticDiagnostics(ctx, delegate, fileName) {
    const errors = delegate(fileName) || [];
    if (ctx.getScriptSourceHelper().isExcluded(fileName))
        return errors;
    const nodes = ctx.findTemplateNodes(fileName);
    const result = [...errors];
    const [schema, schemaErrors] = ctx.getSchemaOrSchemaErrors();
    if (schemaErrors) {
        nodes.forEach(node => {
            schemaErrors.forEach(schemaErrorInfo => {
                const file = node.getSourceFile();
                const start = node.getStart();
                const lengrh = node.getWidth();
                result.push(createSchemaErrorDiagnostic(schemaErrorInfo, file, start, lengrh));
            });
        });
    }
    else if (schema) {
        const diagnosticsAndResolvedInfoList = nodes.map(n => {
            const { text, sourcePosition } = (0, ts_ast_util_1.getSanitizedTemplateText)(n);
            result.push(...(0, gql_ast_util_1.getFragmentsInDocument)(ctx.getGraphQLDocumentNode(text))
                .filter(fragmentDef => ctx.getDuplicaterdFragmentDefinitions().has(fragmentDef.name.value))
                .map(fragmentDef => createDuplicatedFragmentDefinitonsDiagnostic(n.getSourceFile(), sourcePosition, fragmentDef)));
            const { resolvedInfo, resolveErrors } = ctx.resolveTemplateInfo(fileName, n);
            const externalFragments = resolvedInfo
                ? ctx.getExternalFragmentDefinitions(resolvedInfo.combinedText, fileName, resolvedInfo.getSourcePosition(0).pos)
                : [];
            return {
                resolveErrors,
                resolvedTemplateInfo: resolvedInfo,
                diagnostics: resolvedInfo
                    ? (0, graphql_language_service_1.getDiagnostics)(resolvedInfo.combinedText, schema, undefined, undefined, externalFragments)
                    : [],
            };
        });
        diagnosticsAndResolvedInfoList.forEach((info, i) => {
            const node = nodes[i];
            if (!info.resolvedTemplateInfo) {
                info.resolveErrors
                    .filter(re => re.fileName === fileName)
                    .forEach(resolveError => {
                    result.push({
                        category: tsmodule_1.default.DiagnosticCategory.Warning,
                        code: errors_1.ERROR_CODES.templateIsTooComplex.code,
                        messageText: errors_1.ERROR_CODES.templateIsTooComplex.message,
                        file: node.getSourceFile(),
                        start: resolveError.start,
                        length: resolveError.end,
                    });
                });
                return;
            }
            const { diagnostics, resolvedTemplateInfo: { getSourcePosition, convertInnerLocation2InnerPosition }, } = info;
            diagnostics.forEach(d => {
                let length = 0;
                const file = node.getSourceFile();
                try {
                    const { pos: startPositionOfSource, isInOtherExpression } = getSourcePosition(convertInnerLocation2InnerPosition(d.range.start, true));
                    try {
                        const endPositionOfSource = getSourcePosition(convertInnerLocation2InnerPosition(d.range.end, true)).pos;
                        length = endPositionOfSource - startPositionOfSource - 1;
                    }
                    catch (error) {
                        length = 0;
                    }
                    if (isInOtherExpression) {
                        result.push(createIsInOtherExpressionDiagnostic(file, startPositionOfSource, length));
                    }
                    else {
                        result.push(translateDiagnostic(d, file, startPositionOfSource, length));
                    }
                }
                catch (e) {
                    if (e instanceof string_util_1.OutOfRangeError) {
                        // Note:
                        // We can not convertInnerLocation2InnerPosition if semantics diagnostics are located in externalFragments.
                        // In other words, there is no error in the original sanitized template text, so nothing to do.
                        return;
                    }
                    throw e;
                }
            });
        });
    }
    return result;
}
exports.getSemanticDiagnostics = getSemanticDiagnostics;
//# sourceMappingURL=get-semantic-diagnostics.js.map