"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GraphQLLanguageServiceAdapter = void 0;
const graphql_1 = require("graphql");
const tsmodule_1 = __importDefault(require("../tsmodule"));
const ts_ast_util_1 = require("../ts-ast-util");
const gql_ast_util_1 = require("../gql-ast-util");
const get_completion_at_position_1 = require("./get-completion-at-position");
const get_semantic_diagnostics_1 = require("./get-semantic-diagnostics");
const get_quick_info_at_position_1 = require("./get-quick-info-at-position");
const get_definition_at_position_1 = require("./get-definition-at-position");
const get_definition_and_bound_span_1 = require("./get-definition-and-bound-span");
const cache_1 = require("../cache");
class GraphQLLanguageServiceAdapter {
    constructor(_helper, opt) {
        this._helper = _helper;
        this._parsedDocumentCache = new cache_1.LRUCache(500);
        this._logger = () => { };
        if (opt.logger)
            this._logger = opt.logger;
        if (opt.schemaErrors)
            this.updateSchema(opt.schemaErrors, null);
        if (opt.schema)
            this.updateSchema(null, opt.schema);
        this._tagCondition = opt.tag;
        this._removeDuplicatedFragments = opt.removeDuplicatedFragments;
        this._analysisContext = this._createAnalysisContext();
        this._fragmentRegisry = opt.fragmentRegistry;
    }
    getCompletionAtPosition(delegate, ...args) {
        return (0, get_completion_at_position_1.getCompletionAtPosition)(this._analysisContext, delegate, ...args);
    }
    getSemanticDiagnostics(delegate, ...args) {
        return (0, get_semantic_diagnostics_1.getSemanticDiagnostics)(this._analysisContext, delegate, ...args);
    }
    getQuickInfoAtPosition(delegate, ...args) {
        return (0, get_quick_info_at_position_1.getQuickInfoAtPosition)(this._analysisContext, delegate, ...args);
    }
    getDefinitionAndBoundSpan(delegate, ...args) {
        return (0, get_definition_and_bound_span_1.getDefinitionAndBoundSpan)(this._analysisContext, delegate, ...args);
    }
    getDefinitionAtPosition(delegate, ...args) {
        return (0, get_definition_at_position_1.getDefinitionAtPosition)(this._analysisContext, delegate, ...args);
    }
    updateSchema(errors, schema) {
        if (errors) {
            this._schemaErrors = errors;
            this._schema = null;
        }
        else {
            this._schema = schema;
            this._schemaErrors = null;
        }
    }
    _createAnalysisContext() {
        const ctx = {
            debug: msg => this._logger(msg),
            getScriptSourceHelper: () => this._helper,
            getSchema: () => this._schema,
            getSchemaOrSchemaErrors: () => {
                if (!this._schema) {
                    return [null, this._schemaErrors];
                }
                else {
                    return [this._schema, null];
                }
            },
            getGraphQLDocumentNode: text => this._parse(text),
            getGlobalFragmentDefinitions: () => this._fragmentRegisry.getFragmentDefinitions(),
            getGlobalFragmentDefinitionEntry: (name) => {
                const detail = this._fragmentRegisry.getFragmentDefinitionEntryDetail(name);
                if (!detail)
                    return undefined;
                const { node, fileName, extra: { sourcePosition }, } = detail;
                return {
                    fileName,
                    node,
                    position: sourcePosition,
                };
            },
            getExternalFragmentDefinitions: (documentStr, fileName, sourcePosition) => this._fragmentRegisry.getExternalFragments(documentStr, fileName, sourcePosition),
            getDuplicaterdFragmentDefinitions: () => this._fragmentRegisry.getDuplicaterdFragmentDefinitions(),
            findAscendantTemplateNode: (fileName, position) => this._findAscendantTemplateNode(fileName, position),
            findTemplateNodes: fileName => this._findTemplateNodes(fileName),
            resolveTemplateInfo: (fileName, node) => this._resolveTemplateInfo(fileName, node),
        };
        return ctx;
    }
    _findAscendantTemplateNode(fileName, position) {
        const nodeUnderCursor = this._helper.getNode(fileName, position);
        if (!nodeUnderCursor)
            return;
        let templateNode;
        if (tsmodule_1.default.isNoSubstitutionTemplateLiteral(nodeUnderCursor)) {
            templateNode = nodeUnderCursor;
        }
        else if (tsmodule_1.default.isTemplateHead(nodeUnderCursor) && !(0, ts_ast_util_1.isTemplateLiteralTypeNode)(nodeUnderCursor.parent)) {
            templateNode = nodeUnderCursor.parent;
        }
        else if ((tsmodule_1.default.isTemplateMiddle(nodeUnderCursor) || tsmodule_1.default.isTemplateTail(nodeUnderCursor)) &&
            !(0, ts_ast_util_1.isTemplateLiteralTypeNode)(nodeUnderCursor.parent.parent)) {
            templateNode = nodeUnderCursor.parent.parent;
        }
        else {
            return;
        }
        if (!(0, ts_ast_util_1.isTaggedTemplateNode)(templateNode, this._tagCondition)) {
            return;
        }
        return templateNode;
    }
    _findTemplateNodes(fileName) {
        return this._helper.getAllNodes(fileName, node => (0, ts_ast_util_1.getTemplateNodeUnder)(node, this._tagCondition));
    }
    _parse(text) {
        const cached = this._parsedDocumentCache.get(text);
        if (cached)
            return cached;
        try {
            const parsed = (0, graphql_1.parse)(text);
            this._parsedDocumentCache.set(text, parsed);
            return parsed;
        }
        catch {
            return undefined;
        }
    }
    _resolveTemplateInfo(fileName, node) {
        const { resolvedInfo, resolveErrors } = this._helper.resolveTemplateLiteral(fileName, node);
        if (!resolvedInfo)
            return { resolveErrors };
        const documentNode = this._parse(resolvedInfo.combinedText);
        if (!documentNode) {
            // Note:
            // `parse` throws GraphQL syntax error when combinedText is invalid for GraphQL syntax.
            // We don't need handle this error because getDiagnostics method in this class re-checks syntax with graphql-lang-service,
            return { resolvedInfo, resolveErrors };
        }
        const fragmentNames = (0, gql_ast_util_1.getFragmentNamesInDocument)(documentNode);
        if (!this._removeDuplicatedFragments)
            return { resolveErrors, resolvedInfo, fragmentNames };
        const duplicatedFragmentInfoList = (0, gql_ast_util_1.detectDuplicatedFragments)(documentNode);
        const info = duplicatedFragmentInfoList.reduce((acc, fragmentInfo) => {
            return this._helper.updateTemplateLiteralInfo(acc, fragmentInfo);
        }, resolvedInfo);
        return { resolvedInfo: info, resolveErrors };
    }
}
exports.GraphQLLanguageServiceAdapter = GraphQLLanguageServiceAdapter;
//# sourceMappingURL=graphql-language-service-adapter.js.map